import pickle
import inspect
from cryptwood import AES256cryptor
import os
import configparser

# config = configparser.ConfigParser()
# config.read('config.ini')
# DEFAULT_KEY_PATH = config.get('PathConfig','KEY_PATH')

DEFAULT_KEY_PATH = 'HOMEPATH'

class fileManager():

    @staticmethod
    def getKeyPath(projectPath):
        homePath = os.path.expanduser('~')if DEFAULT_KEY_PATH == "HOMEPATH" else DEFAULT_KEY_PATH

        keyPath = os.path.join(homePath,".cryptUserDataKey",(projectPath.replace(os.sep, '_')).replace(":",''))
        return keyPath

    @staticmethod
    def getCiphertextPath(projectPath):
        projectPath = os.path.join(projectPath,".cryptUserData")
        return projectPath

    @staticmethod
    def addToFile(cryptAns, key, iv, path):
        keyPath = fileManager.getKeyPath(path)
        directory = os.path.dirname(keyPath)
        if not os.path.exists(directory):
            os.makedirs(directory)
        with open(keyPath+"key", 'wb') as file:
            file.write(key)
        with open(keyPath + "iv", 'wb') as file:
            file.write(iv)
        with open(fileManager.getCiphertextPath(path), "wb") as file:
            file.write(cryptAns)

class dataCrypter:

    @staticmethod
    def encrypt(userData):
        key = os.urandom(32)
        projectPath = os.path.dirname(inspect.stack()[1].filename)
        dumpData = pickle.dumps(userData)
        cryptAns, iv = AES256cryptor.aes_cbc_encrypt(key=key, plaintext=dumpData)
        fileManager.addToFile(cryptAns,key, iv ,projectPath)

    @staticmethod
    def decrypt():
        projectPath = os.path.dirname(inspect.stack()[1].filename)
        keyPath = fileManager.getKeyPath(projectPath)
        with open(keyPath + "key", "rb") as file:
            key = file.read()
        with open(keyPath + "iv", "rb") as file:
            iv = file.read()
        with open(fileManager.getCiphertextPath(projectPath), "rb") as file:
            userData = file.read()
        dumpData = AES256cryptor.aes_cbc_decrypt(key=key, iv=iv, ciphertext=userData)
        return pickle.loads(dumpData)


from setuptools import setup, find_packages
# import os
# import configparser
# from setuptools.command.install import install

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

# def getCustomPath():
    # while (True):
    #     customPath = input("custom directory to keep key(none for default:current user root directory):")
    #     if customPath == '':
    #         customPath = 'HOMEPATH'
    #         break
    #     if not os.path.exists(customPath):
    #         if os.path.isabs(customPath):
    #             opt = input("path not exists,creat directory(s)?(y/n)")
    #             if (opt == 'y'):
    #                 if not os.path.exists(customPath):
    #                     os.makedirs(customPath)
    #                     break
    #         else:
    #             print(r"illegal key path!(\ for windows /for linux)")
    #     elif os.path.isfile(customPath):
    #         print(r"expected directory path not file path")
    #     else:
    #         break
    # opt = input("key file will stored in " + customPath + os.sep + ".cryptUserDataKey" + "(y/n):")
    # if opt == 'n':
    #     getCustomPath()
    # return customPath

# class CustomInstallCommand(install):
#     def run(self):
#         install.run(self)
#         customPath = getCustomPath()
#         config_file = os.path.join(self.install_lib, 'cryptwood', 'config.ini')
#         open(config_file, 'a')
#         print(config_file)
#         config = configparser.ConfigParser()
#         config.read(config_file)
#         print(config_file)
#         config.add_section('PathConfig')
#         config.set('PathConfig', 'KEY_PATH', customPath)

setup(
    name='cryptwood',
    version='0.1.4',
    license='GPL-3.0',
    description=r"A tool for encrypting sensitive data",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='RedForestLonvor',
    author_email='redforestlonvor@outlook.com',
    url='https://github.com/RedForestLonvor/cryptwood',
    packages=find_packages(),
    install_requires=[
        'cryptography>=3.0'
    ],
    # entry_points={
    #     'console_scripts': [
    #         'cryptree_setup = cryptwood.setup:run_setup'
    #     ]
    # },
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 3',
        'Operating System :: OS Independent',
    ],
    # cmdclass={
    #     'install': CustomInstallCommand,
    # }
)



from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes
from cryptography.hazmat.primitives import padding
import os

def aes_cbc_encrypt(key, plaintext, iv = os.urandom(16)):
    cipher = Cipher(algorithms.AES(key), modes.CBC(iv))
    padder = padding.PKCS7(128).padder()
    padded_plaintext = padder.update(plaintext) + padder.finalize()
    encryptor = cipher.encryptor()
    ciphertext = encryptor.update(padded_plaintext) + encryptor.finalize()
    return ciphertext,iv

def aes_cbc_decrypt(key, iv, ciphertext):
    cipher = Cipher(algorithms.AES(key), modes.CBC(iv))
    decryptor = cipher.decryptor()
    plaintext = decryptor.update(ciphertext) + decryptor.finalize()
    unpadder = padding.PKCS7(128).unpadder()
    plaintext = unpadder.update(plaintext) + unpadder.finalize()
    return plaintext